#! /usr/bin/env python

import unittest


LINE_END = '\r\n'
BLANK_LINE = '\r\n\r\n'

RECOG_METHOD_NAMES = ['DEFINE-GRAMMAR',
                      'RECOGNIZE',
                      'INTERPRET',
                      'GET-RESULT',
                      'START-INPUT-TIMERS',
                      'STOP']

RECOG_EVENT_NAMES = ['START-OF-INPUT',
                     'RECOGNITION-COMPLETE',
                     'INTERPRETATION-COMPLETE']

## REQUEST_STATES = ["COMPLETE",
##                   "IN-PROGRESS",
##                   "PENDING"]


class MRCP2_Message(object):
    """MRCPv2 message format:
   generic-message  =    start-line
                         message-header
                         CRLF
                         [ message-body ]

   start-line       =    request-line / response-line / event-line

   message-header   =    1*(generic-header / resource-header)

   resource-header  =    recognizer-header
                    /    synthesizer-header
                    /    recorder-header
                    /    verifier-header
    """

    def __init__(self, message=''):
        self.start_line = ''
        self.mrcp_version = "MRCP/2.0"
        self.observed_message_length = 0
        self.message_type = '' # request | response | event
        self.request_id = 0
        self.request_state = '' # 
        self.status_code = 000 # see mrcpv2#section-5.4
        ### working here now
        ### what to do about method/event name

# request:  request-id               method-name 
# response: request-id request-state status-code 
# event:    request-id request-state event-name   

        self.message_header = ''
        self.generic_header = ''
        self.resource_header = ''
        self.message_body = None

        if message:
            self.parse(message)

    def parse(self, message):
        self.observed_message_length = len(message)
        if BLANK_LINE in message:
            idx = message.index(BLANK_LINE)
            header = message[:idx]
            body = message[idx+4:]
            idx = header.index(LINE_END)
            start_line = header[:idx]
            message_header = header[idx+2:]
            self.parse_start_line(start_line)
            self.parse_message_header(message_header)
            self.parse_body(body)
        else:
            raise StandardError('No head/body separation.')

    def parse_start_line(self, line):
        """
        start-line     =  mrcp-version SP message-length SP (x-line) CRLF

        request-line   =  method-name SP request-id
        response-line  =  request-id  SP status-code SP request-state
        event-line     =  event-name  SP request-id  SP request-state

        event-name    = mrcp resource event eg RECOGNITION-COMPLETE
        method-name   = mrcp resource method eg DEFINE-GRAMMAR
        request-id    = 1*DIGIT
        request-state = "COMPLETE" | "IN-PROGRESS" | "PENDING"
        status-code   = 3*DIGIT status code (see mrcpv2.10#Section-5.4
        """
        tokens = line.split(' ')
        self.validate_mrcp_version(tokens.pop(0))
        self.validate_message_length(tokens.pop(0))
        if len(tokens) == 2:
            self.message_type = 'request'
            self.validate_request_line(tokens)
        elif tokens[0] in RECOG_EVENT_NAMES:
            self.message_type = 'event'
            self.validate_event_line(tokens)
        else:
            self.message_type = 'response'
            self.validate_response_line(tokens)

    def validate_mrcp_version(self, item):
        if item != self.mrcp_version:
            # todo: encoding!?
            # todo: using exceptions/error messages
            raise StandardError('Invalid MRCP Version: %s != %s.' \
                            % (item, self.mrcp_version))

    def validate_message_length(self, item):
        if int(item) != self.observed_message_length:
            raise StandardError('Stated and observed message lengths differ.')

    def validate_request_line(self, tokens):
        'request-line   =  method-name SP request-id'
        if tokens[0] in RECOG_METHOD_NAMES:
            pass
        else:
            raise StandardError('Unrecognised SpeechRecog method requested.')
            

    def validate_response_line(self, tokens):
        'response-line  =  request-id  SP status-code SP request-state'
        pass

    def validate_event_line(self, tokens):
        'event-line     =  event-name  SP request-id  SP request-state'
        pass

        
    def parse_message_header(self, head):
        pass

    def parse_body(self, body):
        pass


class MRCP2_MessageTests(unittest.TestCase):

    def setUp(self):
        self.fn = 'examples/mrcp_message_1.txt'
        self.indata = open(self.fn).read()

    def testMRCP2_Message(self):
        self.failUnless(MRCP2_Message(self.indata))

def main():
    unittest.main()

if __name__ == '__main__':
    main()
